@@ -26,6 +26,18 @@ try:
 except ImportError:
     pass
 
+def wrap(text, length):
+    words = text.split()
+    lines = []
+    line = ''
+    for w in words:
+        if len(w) + len(line) > length:
+            lines.append(line)
+            line = ''
+        line = line + w + ' '
+        if w is words[-1]: lines.append(line)
+    return '\n'.join(lines)
+
 
 class PlotDialog(QtGui.QDialog):
     """Dialog to plot arbitrary NeXus data in one or two dimensions"""
@@ -38,22 +50,19 @@ class PlotDialog(QtGui.QDialog):
         self.dims = len(self.node.shape)
         
         if isinstance(node, NXfield):
-            plotlayout = QtGui.QHBoxLayout()
-            if self.dims == 1:
-                plotlabel = [QtGui.QLabel("Choose x-axis: ")]
-                self.plotbox = [self.axis_box(0)]
-                plotlayout.addWidget(plotlabel[0])
-                plotlayout.addWidget(self.plotbox[0])
-            else:
-                plotlabel = [QtGui.QLabel("Choose x-axis: "),
-                             QtGui.QLabel("Choose y-axis: ")]
-                self.plotbox = [self.axis_box(0), self.axis_box(1)]
-                plotlayout.addWidget(plotlabel[1])
-                plotlayout.addWidget(self.plotbox[1])
-                plotlayout.addWidget(plotlabel[0])
-                plotlayout.addWidget(self.axis_box[0])
+            axis_layout = [QtGui.QHBoxLayout()]
+            axis_label = [QtGui.QLabel("Choose Axis 0: ")]
+            self.axis_boxes = [self.axis_box(0)]
+            axis_layout[0].addWidget(axis_label[0])
+            axis_layout[0].addWidget(self.axis_boxes[0])
+            for axis in range(1, self.dims):
+                axis_layout.append(QtGui.QHBoxLayout())
+                axis_label.append(QtGui.QLabel("Choose Axis %s: " % axis))
+                self.axis_boxes.append(self.axis_box(axis))
+                axis_layout[axis].addWidget(axis_label[axis])
+                axis_layout[axis].addWidget(self.axis_boxes[axis])
         else:
-            plotlayout = None
+            raise NeXusError('Item not plottable')
 
         buttonbox = QtGui.QDialogButtonBox(self)
         buttonbox.setOrientation(QtCore.Qt.Horizontal)
@@ -63,20 +72,21 @@ class PlotDialog(QtGui.QDialog):
         buttonbox.rejected.connect(self.reject)
 
         layout = QtGui.QVBoxLayout()
-        if plotlayout:
-            layout.addLayout(plotlayout)
+        for axis in range(self.dims):
+            layout.addLayout(axis_layout[axis])
         layout.addWidget(buttonbox) 
         self.setLayout(layout)
 
         self.setWindowTitle("Plot NeXus Field")
 
     def axis_box(self, axis):
-        plotbox = QtGui.QComboBox()
+        box = QtGui.QComboBox()
         for node in self.node.nxgroup.entries.values():
             if node is not self.node and self.check_axis(node, axis):
-                plotbox.addItem(node.nxname)
-        plotbox.insertSeparator(0)
-        plotbox.insertItem(0,'NXfield index')
+                box.addItem(node.nxname)
+        if box.count() > 0:
+            box.insertSeparator(0)
+        box.insertItem(0,'NXfield index')
         if 'axes' in self.node.attrs:
             from nexpy.api.nexus.tree import _readaxes
             default_axis = _readaxes(self.node.axes)[axis]
@@ -86,10 +96,12 @@ class PlotDialog(QtGui.QDialog):
             default_axis = None
         if default_axis:
             try:
-                plotbox.setCurrentIndex(plotbox.findText(default_axis))
+                box.setCurrentIndex(box.findText(default_axis))
             except Exception:
                 pass
-        return plotbox
+        else:
+            box.setCurrentIndex(box.findText('NXfield index'))
+        return box
 
     def check_axis(self, node, axis):
         if len(node.shape) > 1:
@@ -103,17 +115,15 @@ class PlotDialog(QtGui.QDialog):
         return False
 
     def get_axis(self, axis):
-        axis_name = self.plotbox[axis].currentText()
+        axis_name = self.axis_boxes[axis].currentText()
         if axis_name == 'NXfield index':
-            return NXfield(range(1, self.node.size+1), name='index')
+            return NXfield(range(self.node.shape[axis]), 
+                           name='index_%s' % axis)
         else:
             return self.node.nxgroup.entries[axis_name]
 
     def get_axes(self):
-        if self.dims == 1:
-            return [self.get_axis(0)]
-        else:
-            return [self.get_axis(0), self.get_axis(1)]
+        return [self.get_axis(axis) for axis in range(self.dims)]
 
     def accept(self):
         data = NXdata(self.node, self.get_axes())
@@ -124,6 +134,78 @@ class PlotDialog(QtGui.QDialog):
         QtGui.QDialog.reject(self)
 
     
+class LimitDialog(QtGui.QDialog):
+    """Dialog to set plot window limits
+    
+    This is useful when it is desired to set the limits outside the data limits. 
+    """
+ 
+    def __init__(self, parent=None):
+
+        QtGui.QDialog.__init__(self, parent)
+ 
+        from nexpy.gui.plotview import plotview
+
+        self.plotview = plotview
+        
+        layout = QtGui.QVBoxLayout()
+        xmin_layout = QtGui.QHBoxLayout()
+        xmin_layout.addWidget(QtGui.QLabel('xmin'))
+        self.xmin_box = self.textbox()
+        self.xmin_box.setValue(plotview.plot.xaxis.min)
+        xmin_layout.addWidget(self.xmin_box)
+        layout.addLayout(xmin_layout)
+
+        xmax_layout = QtGui.QHBoxLayout()
+        xmax_layout.addWidget(QtGui.QLabel('xmax'))
+        self.xmax_box = self.textbox()
+        self.xmax_box.setValue(plotview.plot.xaxis.max)
+        xmax_layout.addWidget(self.xmax_box)
+        layout.addLayout(xmax_layout)
+
+        ymin_layout = QtGui.QHBoxLayout()
+        ymin_layout.addWidget(QtGui.QLabel('ymin'))
+        self.ymin_box = self.textbox()
+        self.ymin_box.setValue(plotview.plot.yaxis.min)
+        ymin_layout.addWidget(self.ymin_box)
+        layout.addLayout(ymin_layout)
+
+        ymax_layout = QtGui.QHBoxLayout()
+        ymax_layout.addWidget(QtGui.QLabel('ymax'))
+        self.ymax_box = self.textbox()
+        self.ymax_box.setValue(plotview.plot.yaxis.max)
+        ymax_layout.addWidget(self.ymax_box)
+        layout.addLayout(ymax_layout)
+
+        buttonbox = QtGui.QDialogButtonBox(self)
+        buttonbox.setOrientation(QtCore.Qt.Horizontal)
+        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
+                                     QtGui.QDialogButtonBox.Ok)
+        buttonbox.accepted.connect(self.accept)
+        buttonbox.rejected.connect(self.reject)
+
+        layout.addWidget(buttonbox) 
+        self.setLayout(layout)
+
+        self.setWindowTitle("Limit axes")
+
+    def textbox(self):
+        from nexpy.gui.plotview import NXTextBox
+        textbox = NXTextBox()
+        textbox.setAlignment(QtCore.Qt.AlignRight)
+        textbox.setFixedWidth(75)
+        return textbox
+
+    def accept(self):
+        xmin, xmax = self.xmin_box.value(), self.xmax_box.value() 
+        ymin, ymax = self.ymin_box.value(), self.ymax_box.value() 
+        self.plotview.set_limits(xmin, xmax, ymin, ymax)
+        QtGui.QDialog.accept(self)
+        
+    def reject(self):
+        QtGui.QDialog.reject(self)
+
+    
 class AddDialog(QtGui.QDialog):
     """Dialog to add a NeXus node"""
 
@@ -137,8 +219,6 @@ class AddDialog(QtGui.QDialog):
         self.node = node
 
         class_layout = QtGui.QHBoxLayout()
-        class_button = QtGui.QPushButton("Add")
-        class_button.clicked.connect(self.select_class)
         self.class_box = QtGui.QComboBox()
         if isinstance(self.node, NXgroup):
             names = ['NXgroup', 'NXfield']
@@ -146,8 +226,10 @@ class AddDialog(QtGui.QDialog):
             names = ['NXattr']
         for name in names:
             self.class_box.addItem(name)
-        class_layout.addWidget(class_button)
+        self.class_button = QtGui.QPushButton("Add")
+        self.class_button.clicked.connect(self.select_class)
         class_layout.addWidget(self.class_box)
+        class_layout.addWidget(self.class_button)
         class_layout.addStretch()       
 
         if isinstance(self.node, NXfield):
@@ -171,8 +253,12 @@ class AddDialog(QtGui.QDialog):
         self.class_name = self.class_box.currentText()
         if self.class_name == "NXgroup":
             self.layout.insertLayout(1, self.define_grid("NXgroup"))
-        else:
+        elif self.class_name == "NXfield":
             self.layout.insertLayout(1, self.define_grid("NXfield"))
+        else:
+            self.layout.insertLayout(1, self.define_grid("NXattr"))
+        self.class_button.setDisabled(True)
+        self.class_box.setDisabled(True)
 
     def define_grid(self, class_name):
         grid = QtGui.QGridLayout()
@@ -184,19 +270,68 @@ class AddDialog(QtGui.QDialog):
         self.name_box = QtGui.QLineEdit()
         self.name_box.setAlignment(QtCore.Qt.AlignLeft)
         if class_name == "NXgroup":
-            type_label = QtGui.QLabel()
-            type_label.setAlignment(QtCore.Qt.AlignLeft)
-            type_label.setText("Group Class:")
-            self.type_box = QtGui.QComboBox()
-            from nexpy.api.nexus.tree import nxclasses
-            for name in nxclasses:
-                if name != 'NXroot':
-                    self.type_box.addItem(name)
-            self.type_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
-            grid.addWidget(type_label, 0, 0)
-            grid.addWidget(self.type_box, 0, 1)
+            combo_label = QtGui.QLabel()
+            combo_label.setAlignment(QtCore.Qt.AlignLeft)
+            combo_label.setText("Group Class:")
+            self.combo_box = QtGui.QComboBox()
+            self.combo_box.currentIndexChanged.connect(self.select_combo)
+            from nexpy.gui.consoleapp import _mainwindow
+            standard_groups = sorted(list(set([g for g in 
+                              _mainwindow.nxclasses[self.node.nxclass][2]])))
+            for name in standard_groups:
+                self.combo_box.addItem(name)
+                self.combo_box.setItemData(self.combo_box.count()-1, 
+                    wrap(_mainwindow.nxclasses[name][0], 40),
+                    QtCore.Qt.ToolTipRole)
+            self.combo_box.insertSeparator(self.combo_box.count())
+            other_groups = sorted([g for g in _mainwindow.nxclasses if g not in
+                                   standard_groups])
+            for name in other_groups:
+                self.combo_box.addItem(name)
+                self.combo_box.setItemData(self.combo_box.count()-1, 
+                    wrap(_mainwindow.nxclasses[name][0], 40),
+                    QtCore.Qt.ToolTipRole)
+            self.combo_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+            grid.addWidget(combo_label, 0, 0)
+            grid.addWidget(self.combo_box, 0, 1)
             grid.addWidget(name_label, 1, 0)
             grid.addWidget(self.name_box, 1, 1)
+        elif class_name == "NXfield":
+            combo_label = QtGui.QLabel()
+            combo_label.setAlignment(QtCore.Qt.AlignLeft)
+            self.combo_box = QtGui.QComboBox()
+            self.combo_box.currentIndexChanged.connect(self.select_combo)
+            from nexpy.gui.consoleapp import _mainwindow
+            fields = sorted(list(set([g for g in 
+                            _mainwindow.nxclasses[self.node.nxclass][1]])))
+            for name in fields:
+                self.combo_box.addItem(name)
+                self.combo_box.setItemData(self.combo_box.count()-1, 
+                    wrap(_mainwindow.nxclasses[self.node.nxclass][1][name][2], 40),
+                    QtCore.Qt.ToolTipRole)
+            self.combo_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+            grid.addWidget(name_label, 0, 0)
+            grid.addWidget(self.name_box, 0, 1)
+            grid.addWidget(self.combo_box, 0, 2)
+            value_label = QtGui.QLabel()
+            value_label.setAlignment(QtCore.Qt.AlignLeft)
+            value_label.setText("Value:")
+            self.value_box = QtGui.QLineEdit()
+            self.value_box.setAlignment(QtCore.Qt.AlignLeft)
+            grid.addWidget(value_label, 1, 0)
+            grid.addWidget(self.value_box, 1, 1)
+            type_label = QtGui.QLabel()
+            type_label.setAlignment(QtCore.Qt.AlignLeft)
+            type_label.setText("Datatype:")
+            self.type_box = QtGui.QComboBox()
+            for name in self.data_types:
+                self.type_box.addItem(name)
+            self.type_box.insertSeparator(0)
+            self.type_box.insertItem(0, 'auto')
+            self.type_box.setCurrentIndex(0)
+            self.type_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+            grid.addWidget(type_label, 2, 0)
+            grid.addWidget(self.type_box, 2, 1)
         else:
             grid.addWidget(name_label, 0, 0)
             grid.addWidget(self.name_box, 0, 1)
@@ -222,9 +357,17 @@ class AddDialog(QtGui.QDialog):
         grid.setColumnMinimumWidth(1, 200)
         return grid
 
+    def select_combo(self):
+        self.set_name(self.combo_box.currentText())
+    
     def get_name(self):
         return self.name_box.text()
 
+    def set_name(self, name):
+        if self.class_name == 'NXgroup':
+            name = name[2:]
+        self.name_box.setText(name)
+
     def get_value(self):
         value = self.value_box.text()
         if value:
@@ -241,11 +384,14 @@ class AddDialog(QtGui.QDialog):
             return None
 
     def get_type(self):
-        dtype = self.type_box.currentText()
-        if dtype == "auto":
-            return None
+        if self.class_name == 'NXgroup':
+            return self.combo_box.currentText()
         else:
-            return dtype 
+            dtype = self.type_box.currentText()
+            if dtype == "auto":
+                return None
+            else:
+                return dtype 
 
     def accept(self):
         name = self.get_name()
@@ -291,8 +437,20 @@ class InitializeDialog(QtGui.QDialog):
         name_label.setText("Name:")
         self.name_box = QtGui.QLineEdit()
         self.name_box.setAlignment(QtCore.Qt.AlignLeft)
+        self.combo_box = QtGui.QComboBox()
+        self.combo_box.currentIndexChanged.connect(self.select_combo)
+        from nexpy.gui.consoleapp import _mainwindow
+        fields = sorted(list(set([g for g in 
+                        _mainwindow.nxclasses[self.node.nxclass][1]])))
+        for name in fields:
+            self.combo_box.addItem(name)
+            self.combo_box.setItemData(self.combo_box.count()-1, 
+                wrap(_mainwindow.nxclasses[self.node.nxclass][1][name][2], 40),
+                QtCore.Qt.ToolTipRole)
+        self.combo_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
         grid.addWidget(name_label, 0, 0)
         grid.addWidget(self.name_box, 0, 1)
+        grid.addWidget(self.combo_box, 0, 2)
         type_label = QtGui.QLabel()
         type_label.setAlignment(QtCore.Qt.AlignLeft)
         type_label.setText("Datatype:")
@@ -324,9 +482,15 @@ class InitializeDialog(QtGui.QDialog):
         self.layout.addWidget(buttonbox) 
         self.setLayout(self.layout)
 
+    def select_combo(self):
+        self.set_name(self.combo_box.currentText())
+    
     def get_name(self):
         return self.name_box.text()
 
+    def set_name(self, name):
+        self.name_box.setText(name)
+
     def get_type(self):
         dtype = self.type_box.currentText()
         return dtype 
@@ -334,9 +498,17 @@ class InitializeDialog(QtGui.QDialog):
     def get_shape(self):
         import ast
         try:
-            return ast.literal_eval(self.shape_box.text())
+            shape = ast.literal_eval(self.shape_box.text())
+            try:
+                it = iter(shape)
+                return shape
+            except TypeError:
+                if isinstance(shape, int):
+                    return (shape,)
+                else:
+                    raise NeXusError('Invalid shape')
         except ValueError:
-            return None
+            raise NeXusError('Invalid shape')
 
     def accept(self):
         name = self.get_name()
@@ -352,6 +524,111 @@ class InitializeDialog(QtGui.QDialog):
         QtGui.QDialog.reject(self)
 
     
+class RenameDialog(QtGui.QDialog):
+    """Dialog to rename a NeXus node"""
+
+    def __init__(self, node, parent=None):
+
+        QtGui.QDialog.__init__(self, parent)
+
+        self.node = node
+
+        self.setWindowTitle("Rename NeXus data")
+
+        buttonbox = QtGui.QDialogButtonBox(self)
+        buttonbox.setOrientation(QtCore.Qt.Horizontal)
+        buttonbox.setStandardButtons(QtGui.QDialogButtonBox.Cancel |
+                                     QtGui.QDialogButtonBox.Ok)
+        buttonbox.accepted.connect(self.accept)
+        buttonbox.rejected.connect(self.reject)
+
+        self.layout = QtGui.QVBoxLayout()
+        self.layout.addLayout(self.define_grid())
+        self.layout.addWidget(buttonbox) 
+        self.setLayout(self.layout)
+
+    def define_grid(self):
+        grid = QtGui.QGridLayout()
+        grid.setSpacing(10)
+        name_label = QtGui.QLabel()
+        name_label.setAlignment(QtCore.Qt.AlignLeft)
+        name_label.setText("New Name:")
+        self.name_box = QtGui.QLineEdit(self.node.nxname)
+        self.name_box.setAlignment(QtCore.Qt.AlignLeft)
+        grid.addWidget(name_label, 0, 0)
+        grid.addWidget(self.name_box, 0, 1)
+        self.combo_box = None
+        if isinstance(self.node, NXgroup) and self.node.nxclass != 'NXroot':
+            combo_label = QtGui.QLabel()
+            combo_label.setAlignment(QtCore.Qt.AlignLeft)
+            combo_label.setText("New Class:")
+            self.combo_box = QtGui.QComboBox()
+            from nexpy.gui.consoleapp import _mainwindow
+            parent_class = self.node.nxgroup.nxclass
+            standard_groups = sorted(list(set([g for g in 
+                          _mainwindow.nxclasses[parent_class][2]])))
+            for name in standard_groups:
+                self.combo_box.addItem(name)
+                self.combo_box.setItemData(self.combo_box.count()-1, 
+                    wrap(_mainwindow.nxclasses[name][0], 40),
+                    QtCore.Qt.ToolTipRole)
+            self.combo_box.insertSeparator(self.combo_box.count())
+            other_groups = sorted([g for g in _mainwindow.nxclasses 
+                                   if g not in standard_groups])
+            for name in other_groups:
+                self.combo_box.addItem(name)
+                self.combo_box.setItemData(self.combo_box.count()-1, 
+                    wrap(_mainwindow.nxclasses[name][0], 40),
+                    QtCore.Qt.ToolTipRole)
+            self.combo_box.insertSeparator(self.combo_box.count())
+            self.combo_box.addItem('NXgroup')
+            self.combo_box.setCurrentIndex(self.combo_box.findText(self.node.nxclass))
+            self.combo_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+            grid.addWidget(combo_label, 1, 0)
+            grid.addWidget(self.combo_box, 1, 1)
+        else:
+            parent_class = self.node.nxgroup.nxclass
+            if parent_class != 'NXroot' or parent_class != 'NXtree':
+                combo_label = QtGui.QLabel()
+                combo_label.setAlignment(QtCore.Qt.AlignLeft)
+                combo_label.setText("Valid Fields:")
+                self.combo_box = QtGui.QComboBox()
+                self.combo_box.currentIndexChanged.connect(self.set_name)
+                from nexpy.gui.consoleapp import _mainwindow
+                fields = sorted(list(set([g for g in 
+                            _mainwindow.nxclasses[parent_class][1]])))
+                for name in fields:
+                    self.combo_box.addItem(name)
+                    self.combo_box.setItemData(self.combo_box.count()-1, 
+                        wrap(_mainwindow.nxclasses[parent_class][1][name][2], 40),
+                        QtCore.Qt.ToolTipRole)
+                self.combo_box.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+                grid.addWidget(self.combo_box, 0, 2)
+        grid.setColumnMinimumWidth(1, 200)
+        return grid
+
+    def get_name(self):
+        return self.name_box.text()
+
+    def set_name(self):
+        self.name_box.setText(self.combo_box.currentText())
+
+    def get_class(self):
+        return self.combo_box.currentText()
+
+    def accept(self):
+        name = self.get_name()
+        if name:
+            self.node.rename(name)
+        if isinstance(self.node, NXgroup):
+            if self.combo_box:
+                self.node.nxclass = self.get_class()
+        QtGui.QDialog.accept(self)
+        
+    def reject(self):
+        QtGui.QDialog.reject(self)
+
+    
 class RemoveDialog(QtGui.QDialog):
     """Dialog to remove a NeXus node from the tree"""
  
