@@ -19,12 +19,14 @@ of a Matplotlib plotting pane and a tree view for displaying NeXus data.
 # Imports
 #-----------------------------------------------------------------------------
 
-import imp          #@UnusedImports
+import glob
+import imp
 import json
-import os           #@UnusedImports
-import re           #@UnusedImports
-import sys          #@UnusedImports
+import os
+import re
+import sys
 import webbrowser
+import xml.etree.ElementTree as ET
 from threading import Thread
 
 from PySide import QtGui, QtCore
@@ -39,7 +41,7 @@ def background(f):
 # local imports
 from treeview import NXTreeView
 from plotview import NXPlotView
-from datadialogs import *           #@UnusedWildImports
+from datadialogs import *
 from nexpy.api.nexus.tree import nxload, NeXusError, NXFile, NXlink
 #from nexpy.api.nexus.tree import NXFile, NXgroup, NXfield, NXroot, NXentry, NXlink
 
@@ -153,6 +155,7 @@ class MainWindow(QtGui.QMainWindow):
         self.setCentralWidget(mainwindow)
 
         self.import_path = pkg_resources.resource_filename('nexpy','readers')
+        self.input_base_classes()
 
         self.init_menu_bar()
 
@@ -167,9 +170,7 @@ class MainWindow(QtGui.QMainWindow):
         """ Called when you quit NeXpy or close the main window.
         """
         title = self.window().windowTitle()
-        cancel = QtGui.QMessageBox.Cancel
-        okay = QtGui.QMessageBox.Ok         # TODO: unused
-        
+        cancel = QtGui.QMessageBox.Cancel        
         msg = "Are you sure you want to quit NeXpy?"
         close = QtGui.QPushButton("&Quit", self)
         close.setShortcut('Q')
@@ -403,19 +404,19 @@ class MainWindow(QtGui.QMainWindow):
             )
         self.add_menu_action(self.data_menu, self.rename_action, True)  
 
-        self.copy_action=QtGui.QAction("Copy Data",
+        self.copydata_action=QtGui.QAction("Copy Data",
             self,
             shortcut=QtGui.QKeySequence("Ctrl+Shift+C"),
             triggered=self.copy_data
             )
-        self.add_menu_action(self.data_menu, self.copy_action, True)  
+        self.add_menu_action(self.data_menu, self.copydata_action, True)  
 
-        self.paste_action=QtGui.QAction("Paste Data",
+        self.pastedata_action=QtGui.QAction("Paste Data",
             self,
             shortcut=QtGui.QKeySequence("Ctrl+Shift+V"),
             triggered=self.paste_data
             )
-        self.add_menu_action(self.data_menu, self.paste_action, True)  
+        self.add_menu_action(self.data_menu, self.pastedata_action, True)  
 
         self.pastelink_action=QtGui.QAction("Paste As Link",
             self,
@@ -744,10 +745,8 @@ class MainWindow(QtGui.QMainWindow):
                 node = self.treeview.getnode()
                 if node:
                     if node.nxfilemode != 'r' or isinstance(node, NXroot):
-                        name, ok = QtGui.QInputDialog.getText(self, 'Rename Data',
-                                       'New Name:', text=node.nxname)
-                        if ok:
-                            node.rename(name)
+                        dialog = RenameDialog(node, self)
+                        dialog.show()
                     else:
                         raise NeXusError("NeXus file is locked")
         except NeXusError as error:
@@ -793,7 +792,7 @@ class MainWindow(QtGui.QMainWindow):
                     dialog = DeleteDialog(node, self)
                     dialog.show()
                 else:   
-                    raise NeXusError("NeXus file is locked") 
+                    raise NeXusError("NeXus file is locked")
         except NeXusError as error:
             report_error("Deleting Data", error)
 
@@ -854,7 +853,56 @@ class MainWindow(QtGui.QMainWindow):
         msgBox.setStandardButtons(QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel)
         msgBox.setDefaultButton(QtGui.QMessageBox.Ok)
         return msgBox.exec_()
-       
+
+    def input_base_classes(self):
+        base_class_path = pkg_resources.resource_filename(
+                              'nexpy','definitions/base_classes')
+        nxdl_files = map(os.path.basename, 
+            glob.glob(os.path.join(base_class_path,'*.nxdl.xml')))
+        pattern = re.compile(r'[\t\n ]+')
+        self.nxclasses = {}
+        for nxdl_file in nxdl_files:
+            class_name = nxdl_file.split('.')[0]
+            xml_root = ET.parse(os.path.join(base_class_path, nxdl_file)).getroot()
+            class_doc = ''
+            class_groups = {}
+            class_fields = {}
+            for child in xml_root:
+                name = dtype = units = doc = ''
+                if child.tag.endswith('doc'):
+                    try:
+                        class_doc = re.sub(pattern, ' ', child.text).strip()
+                    except TypeError:
+                        pass
+                if child.tag.endswith('field'):
+                    try:
+                        name = child.attrib['name']
+                        dtype = child.attrib['type']
+                        units = child.attrib['units']
+                    except KeyError:
+                        pass
+                    for element in child:
+                        if element.tag.endswith('doc'):
+                            try:
+                                doc = re.sub(pattern, ' ', element.text).strip()
+                            except TypeError:
+                                pass
+                    class_fields[name] = (dtype, units, doc)
+                elif child.tag.endswith('group'):
+                    try:
+                        dtype = child.attrib['type']
+                        name = child.attrib['name']
+                    except KeyError:
+                        pass
+                    for element in child:
+                        if element.tag.endswith('doc'):
+                            try:
+                                doc = re.sub(pattern, ' ', element.text).strip()
+                            except TypeError:
+                                pass
+                    class_groups[dtype] = (name, doc)
+            self.nxclasses[class_name] = (class_doc, class_fields, class_groups)
+
     def _make_dynamic_magic(self,magic):
         """Return a function `fun` that will execute `magic` on the console.
 
@@ -1056,6 +1104,18 @@ class MainWindow(QtGui.QMainWindow):
 
         self.window_separator = self.window_menu.addSeparator()
 
+        self.limit_action=QtGui.QAction("Change Plot Limits",
+            self,
+            triggered=self.limit_axes
+            )
+        self.add_menu_action(self.window_menu, self.limit_action)
+
+        self.reset_limit_action=QtGui.QAction("Reset Plot Limits",
+            self,
+            triggered=self.reset_axes
+            )
+        self.add_menu_action(self.window_menu, self.reset_limit_action)
+
         self.panel_action=QtGui.QAction("Show Projection Panel",
             self,
             triggered=self.show_projection_panel
@@ -1097,6 +1157,21 @@ class MainWindow(QtGui.QMainWindow):
             plotview.ptab.panel.update_limits()
             plotview.ptab.panel.show()
     
+    def limit_axes(self):
+        try:
+            from nexpy.gui.plotview import plotview
+            dialog = LimitDialog(self)
+            dialog.exec_()
+        except NeXusError as error:
+            report_error("Changing Plot Limits", error)
+    
+    def reset_axes(self):
+        try:
+            from nexpy.gui.plotview import plotview
+            plotview.reset_limits()
+        except NeXusError as error:
+            report_error("Resetting Plot Limits", error)
+    
     def init_help_menu(self):
         # please keep the Help menu in Mac Os even if empty. It will
         # automatically contain a search field to search inside menus and
