@@ -210,7 +210,7 @@ class NXPlotView(QtGui.QWidget):
         #Initialize the plotting window with a token plot
         self.plot.plot(NXdata(signal=NXfield([0,1], name='y'), 
                        axes=NXfield([0,1], name='x')), fmt='wo', mec='w')
-        
+
 #        self.grid_cb = QtGui.QCheckBox("Show &Grid")
 #        self.grid_cb.setChecked(False)
 #        self.grid_cb.stateChanged.connect(self.on_draw)
@@ -262,6 +262,24 @@ class NXPlotView(QtGui.QWidget):
     def redraw(self):
         self.canvas.draw_idle()
 
+    def set_limits(self, xmin=None, xmax=None, ymin=None, ymax=None):
+        if xmin:
+            self.plot.xaxis.min = xmin
+        if xmax:
+            self.plot.xaxis.max = xmax
+        if ymin:
+            self.plot.yaxis.min = ymin
+        if ymax:
+            self.plot.yaxis.max = ymax
+        self.plot.update_tabs()
+
+    def reset_limits(self):
+        self.plot.xaxis.min = self.plot.xaxis.orig_min
+        self.plot.xaxis.max = self.plot.xaxis.orig_max
+        self.plot.yaxis.min = self.plot.yaxis.orig_min
+        self.plot.yaxis.max = self.plot.yaxis.orig_max
+        self.plot.update_tabs()
+
     def close_view(self):
         self.remove_menu_action()
         Gcf.destroy(self.number)
@@ -406,6 +424,11 @@ class NXPlot(object):
         else:
             self.init_tabs()
 
+        self.xaxis.orig_min = self.xaxis.min
+        self.xaxis.orig_max = self.xaxis.max
+        self.yaxis.orig_min = self.yaxis.min
+        self.yaxis.orig_max = self.yaxis.max
+
     def plot1D(self, fmt, over=False, **opts):
 
         mpl.interactive(False)
@@ -643,8 +666,8 @@ class NXPlot(object):
         ax = self.figure.gca()
         ax.set_xlim(self.xaxis.get_limits())
         ax.set_ylim(self.yaxis.get_limits())
-        self.canvas.draw_idle()
         self.otab.push_current()
+        self.canvas.draw_idle()
 
     def replot_logs(self):
         ax = self.figure.gca()
@@ -833,6 +856,7 @@ class NXPlotAxis(object):
     def max_range(self):
         return self.max - self.min
 
+
 class NXReplotSignal(QtCore.QObject):
     
     replot = QtCore.Signal() 
@@ -913,6 +937,7 @@ class NXPlotTab(QtGui.QWidget):
         self.plotview = plotview
 
     def set_axis(self, axis):
+        self.block_signals(True)
         self.plot = self.plotview.plot
         self.axis = axis
         if self.zaxis:
@@ -936,13 +961,12 @@ class NXPlotTab(QtGui.QWidget):
         self.maxbox.old_value = self.maxbox.value()
         self.minbox.block_replot = self.maxbox.block_replot = False
         if not self.zaxis:
-            self.block_signals(True)
             self.set_sliders(self.minbox.minimum(), self.maxbox.maximum())
-            self.block_signals(False)
         if self.axiscombo:
             self.axiscombo.clear()
             self.axiscombo.addItems(self.get_axes())
             self.axiscombo.setCurrentIndex(self.axiscombo.findText(axis.name))
+        self.block_signals(False)
 
     def combobox(self, slot):
         combobox = QtGui.QComboBox()
@@ -951,20 +975,12 @@ class NXPlotTab(QtGui.QWidget):
         combobox.activated.connect(slot)
         return combobox
 
-    def textbox(self, slot):
-        textbox = NXTextBox()
-        textbox.setAlignment(QtCore.Qt.AlignRight)
-        textbox.setFixedWidth(75)
-        textbox.editingFinished.connect(slot)
-        return textbox
-
     def spinbox(self, slot):
         spinbox = NXSpinBox()
         spinbox.setAlignment(QtCore.Qt.AlignRight)
         spinbox.setFixedWidth(100)
         spinbox.setKeyboardTracking(False)
         spinbox.setAccelerated(False)
-        spinbox.editingFinished.connect(slot)
         spinbox.valueChanged[unicode].connect(slot)
         return spinbox
 
@@ -973,7 +989,6 @@ class NXPlotTab(QtGui.QWidget):
         doublespinbox.setAlignment(QtCore.Qt.AlignRight)
         doublespinbox.setFixedWidth(100)
         doublespinbox.setKeyboardTracking(False)
-        doublespinbox.editingFinished.connect(slot)
         doublespinbox.valueChanged[unicode].connect(slot)
         return doublespinbox
 
@@ -1000,6 +1015,7 @@ class NXPlotTab(QtGui.QWidget):
         button.clicked.connect(slot)
         return button
 
+    @QtCore.Slot()
     def read_minbox(self):
         if not self.minbox.isEnabled():
             return
@@ -1019,11 +1035,11 @@ class NXPlotTab(QtGui.QWidget):
         self.minbox.old_value = self.minbox.value()
         self.maxbox.old_value = self.maxbox.value()
 
+    @QtCore.Slot()
     def read_maxbox(self):
         lo, hi = self.minbox.value(), self.maxbox.value()
         if hi == self.maxbox.old_value:
             return
-        replot = False          # TODO: unused variable
         if hi is not None and (hi >= lo or self.axis.locked): 
             self.axis.hi = hi
         else:
@@ -1105,7 +1121,7 @@ class NXPlotTab(QtGui.QWidget):
     def set_lock(self):
         if self.lockbox.isChecked():
             self.axis.locked = True
-            lo, hi = self.axis.get_limits()
+            lo, hi = self.get_limits()
             self.axis.diff = self.maxbox.diff = self.minbox.diff = hi - lo
             self.minbox.setDisabled(True)
         else:
@@ -1127,15 +1143,9 @@ class NXPlotTab(QtGui.QWidget):
         self.block_signals(False)
 
     def reset(self):
-        self.axis.min = np.nanmin(self.axis.data)
-        self.axis.max = np.nanmax(self.axis.data)
         self.minbox.setRange(self.axis.min, self.axis.max)
         self.maxbox.setRange(self.axis.min, self.axis.max)
         self.set_limits(self.axis.min, self.axis.max)
-        if self.name == 'x' or self.name == 'y':
-            self.plot.replot_axes()
-        elif self.name == 'v':
-            self.plot.plot2D()
 
     def get_limits(self):
         return self.minbox.value(), self.maxbox.value()
@@ -1233,6 +1243,7 @@ class NXPlotTab(QtGui.QWidget):
         self.timer.setInterval(self.interval)
         self.timer.start(self.interval)
 
+
 class NXTextBox(QtGui.QLineEdit):
 
     def value(self):
@@ -1315,6 +1326,7 @@ class NXSpinBox(QtGui.QSpinBox):
             self.setValue(self.value() + steps * self.diff)
         else:
             super(NXSpinBox, self).stepBy(steps)
+        self.valueChanged.emit(u'1')
 
 
 class NXDoubleSpinBox(QtGui.QDoubleSpinBox):
@@ -1335,6 +1347,7 @@ class NXDoubleSpinBox(QtGui.QDoubleSpinBox):
             self.setValue(self.value() + steps * self.diff)
         else:
             super(NXDoubleSpinBox, self).stepBy(steps)
+        self.valueChanged.emit(u'1')
 
 
 class NXProjectionTab(QtGui.QWidget):
@@ -1772,7 +1785,6 @@ class NXProjectionPanel(QtGui.QDialog):
         spinbox.setFixedWidth(100)
         spinbox.setKeyboardTracking(False)
         spinbox.setAccelerated(True)
-        spinbox.editingFinished.connect(self.set_limits)
         spinbox.valueChanged[unicode].connect(self.set_limits)
         return spinbox
 
@@ -1825,11 +1837,8 @@ class NXNavigationToolbar(NavigationToolbar):
         super(NXNavigationToolbar, self)._init_toolbar()
 
     def home(self, *args):
-        self.plotview.xtab.reset()
-        self.plotview.ytab.reset()
-        self.plotview.vtab.reset()
-        if self.plotview.ptab.panel:
-            self.plotview.ptab.panel.update_limits()            
+        self.plotview.reset_limits()
+        super(NXNavigationToolbar, self).home()        
 
     def add_data(self):
         keep_data(self.plotview.plot.plotdata)
@@ -1938,7 +1947,6 @@ class NXNavigationToolbar(NavigationToolbar):
 
         self._zoom_mode = None
 
-        self.push_current()
         self.release(event)
 
     def release_pan(self, event):
@@ -1956,14 +1964,18 @@ class NXNavigationToolbar(NavigationToolbar):
         lims = self._views()
         if lims is None: return
         xmin, xmax, ymin, ymax = lims[0]
+        self.plotview.xtab.block_signals(True)
         self.plotview.xtab.axis.set_limits(xmin, xmax)
         self.plotview.xtab.minbox.setValue(xmin)
         self.plotview.xtab.maxbox.setValue(xmax)
         self.plotview.xtab.set_sliders(xmin, xmax)
+        self.plotview.xtab.block_signals(False)
+        self.plotview.ytab.block_signals(True)
         self.plotview.ytab.axis.set_limits(ymin, ymax)
         self.plotview.ytab.minbox.setValue(ymin)
         self.plotview.ytab.maxbox.setValue(ymax)
         self.plotview.ytab.set_sliders(ymin, ymax)
+        self.plotview.ytab.block_signals(False)
 
 #    def set_cursor(self, cursor):
 #        pass
